using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Compiler.Data;

namespace Compiler
{

    public partial class Compiler : Form
    {
        const uint VERSION = 4;
        const byte VIEWCHANGE = 0x08;
        string filename = string.Empty;
        string folderPath = string.Empty;
        GotakuFile quiz = null;
        const int MAX_QUIZ_NUM = 1000;
        System.Reflection.Assembly curasm = null;
        System.Resources.ResourceManager rm = null;

        public Compiler()
        {
            InitializeComponent();
        }


        private void Compiler_Load(object sender, EventArgs e)
        {
            curasm = System.Reflection.Assembly.GetExecutingAssembly();
            rm =
            new System.Resources.ResourceManager(
                "Compiler.Properties.Resources", curasm);

            string[] argv = Environment.GetCommandLineArgs();
            if (argv.Length == 2)
            {
                if (File.Exists(argv[1]))
                {
                    this.filename = argv[1];
                    OpenProcess();
                }
            }
            
        }

        private byte[] GetFileResource(string resourceName)
        {
            return (byte[])rm.GetObject(resourceName);
        }

        private Bitmap GetFileResourceAsBitmap(string resourceName)
        {
            return (Bitmap)rm.GetObject(resourceName);
        }

        /// <summary>
        /// J
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void toolOpen_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog dlg = new OpenFileDialog())
            {
                dlg.CheckFileExists = true;

                dlg.InitialDirectory = @".\";
                dlg.Filter =
                    "5tqt@C(*.5tq)|*.5tq|5txt@C(*.5tx)|*.5tx|qt1`qt8Wt@C(*.qt*)|*.qt*";
                dlg.FilterIndex = 0;

                DialogResult res = dlg.ShowDialog();
                if (res == DialogResult.OK &&
                   File.Exists(dlg.FileName))
                {
                    this.filename = dlg.FileName;
                    OpenProcess();
                }
            }
        }


        private void OpenProcess()
        {
            string extension = Path.GetExtension(filename);
            folderPath = Path.GetDirectoryName(filename);

            extension = extension.ToLower();
            int number = 0;
            if (extension.Equals(".5tq"))
            {
                // ViI쐬
                if (MakeFile(filename, GotakuFile.ReadFileType.GOTAKU_5TQ) == false) return;

            }
            else if (extension.Equals(".5tx"))
            {
                // ViI쐬
                if (MakeFile(filename, GotakuFile.ReadFileType.GOTAKU_5TX) == false) return;
            }
            else if (extension.StartsWith(".qt") && 
                extension.Length == 4 &&
                int.TryParse(extension[3].ToString(), out number) &&
                number >= 1 && 
                number <= 8)
            {
                // ViI쐬
                if (MakeFile(filename, GotakuFile.ReadFileType.GOTAKU_QTN) == false) return;
            }
            else
            {
                MessageBox.Show("Ώۂ̃t@Cł͂܂B");
                return;
            }

            MessageBox.Show("ViI쐬܂B");
        }

        private bool MakeFile(string filename, GotakuFile.ReadFileType type)
        {
            // ViI쐬
            string scenaName = Path.GetFileNameWithoutExtension(filename);

            string scenaFolder = folderPath + "\\" + scenaName;
            if (Directory.Exists(scenaFolder) == false)
            {
                Directory.CreateDirectory(scenaFolder);
            }

            string scenaBmpFolder = scenaFolder + "\\bmp";
            if (Directory.Exists(scenaBmpFolder) == false)
            {
                Directory.CreateDirectory(scenaBmpFolder);
            }

            string scenaJptxFolder = scenaFolder + "\\jptx";
            if (Directory.Exists(scenaJptxFolder) == false)
            {
                Directory.CreateDirectory(scenaJptxFolder);
            }

            // NCYǂݍ
            switch (type)
            {
                case GotakuFile.ReadFileType.GOTAKU_5TQ:
                case GotakuFile.ReadFileType.GOTAKU_5TX:
                case GotakuFile.ReadFileType.GOTAKU_QTN:
                    ReadQuiz(filename, type);
                    break;
                default:
                    MessageBox.Show("肦ȂNCY`łB");
                    return false;
            }

            int quizNum = quiz.QuizDatas.Count;
            if (quizNum == 0)
            {
                MessageBox.Show("NCY݂܂B");
                return false;
            } 
            else if (quizNum >= MAX_QUIZ_NUM)
            {
                MessageBox.Show("NCY" + MAX_QUIZ_NUM + "薢̂̂ɂĂB");
                return false;
            }

            try
            {
                //bmpRs[
                CopyBMP(scenaBmpFolder);

                //EԈႢjptx
                MakeAnswerJptx(scenaJptxFolder);

                // Package
                MakePackage(scenaFolder);

                // Area
                MakeArea(scenaName, scenaFolder);

                // Scenario Summary
                MakeWsm(scenaName, scenaFolder);

            }
            catch (Exception e)
            {
                MessageBox.Show("t@C̓ǂݍ݂Ɏs܂B" + e.Message);
                return false;
            }

            return true;
        }

        private void ReadQuiz(string filename, GotakuFile.ReadFileType type)
        {
            //NCYǂݍ
            try
            {
                quiz = new GotakuFile(type);
                quiz.Read(filename);
            }
            catch (Exception ex)
            {
                MessageBox.Show("NCYǂݍ݂Ɏs܂B" + ex.Message);
                Application.Exit();
            }

        }

        private void CopyBMP(string scenaBmpFolder)
        {
            try
            {
                Bitmap bmp;
                for (int i = 1; i <= 5; i++)
                {
                    bmp = GetFileResourceAsBitmap("_" + i);
                    bmp.Save(scenaBmpFolder + "\\" + i + ".bmp", System.Drawing.Imaging.ImageFormat.Bmp);
                }

                bmp = GetFileResourceAsBitmap("IMAGE_ACTION6");
                bmp.Save(scenaBmpFolder + "\\IMAGE_ACTION6.bmp", System.Drawing.Imaging.ImageFormat.Bmp);

                bmp = GetFileResourceAsBitmap("IMAGE_COMMAND1");
                bmp.Save(scenaBmpFolder + "\\IMAGE_COMMAND1.bmp", System.Drawing.Imaging.ImageFormat.Bmp);

                bmp = GetFileResourceAsBitmap("IMAGE_COMMAND6");
                bmp.Save(scenaBmpFolder + "\\IMAGE_COMMAND6.bmp", System.Drawing.Imaging.ImageFormat.Bmp);

                bmp = GetFileResourceAsBitmap("IMAGE_COMMAND9");
                bmp.Save(scenaBmpFolder + "\\IMAGE_COMMAND9.bmp", System.Drawing.Imaging.ImageFormat.Bmp);
            
            }
            catch (Exception ex)
            {
                MessageBox.Show("\[Xǂݍ߂܂B:" + ex.Message);
                Application.Exit();
            }
        }

        private void MakeAnswerJptx(string scenaJpexFolder)
        {
            // Correct
            MakeJptx(scenaJpexFolder + "\\corr.jptx", 80, 30, 10, "FFFFFF", "I");

            // Wrong
            for (int i = 1; i <= 5; i++)
            {
                MakeJptx(scenaJpexFolder + "\\miss" + i + ".jptx", 80, 30, 10, "FFFFFF",
                    "ԈႢI\r\n " + i);
            }
        }

        private void MakePackage(string scenaFolder)
        {
            // ̑pbP[W͕ωȂ
            WidPackage pack = new WidPackage();
            //try
            //{
            //    pack.Read(GetFileResource("Package2"));
            //}
            //catch (Exception)
            //{
            //    MessageBox.Show("\[Xǂݍ߂܂B:" + "Package2");
            //    Application.Exit();
            //}
            //pack.Write(scenaFolder + "\\Package2.wid");

            //pack = new WidPackage();
            try
            {
                pack.Read(GetFileResource("Package5"));
            }
            catch (Exception)
            {
                MessageBox.Show("\[Xǂݍ߂܂B:" + "Package5");
                Application.Exit();
            }
            pack.Write(scenaFolder + "\\Package5.wid");

            pack = new WidPackage();
            try
            {
                pack.Read(GetFileResource("Package6"));
            }
            catch (Exception)
            {
                MessageBox.Show("\[Xǂݍ߂܂B:" + "Package6");
                Application.Exit();
            }
            pack.Write(scenaFolder + "\\Package6.wid");

            // 胉_IpbP[W
            MakeRandom(scenaFolder);

            // \pbP[W
            // `FbNpbP[W
            MakeQuiz(scenaFolder);
        }

        private void MakeRandom(string scenaFolder)
        {
            // 胉_IpbP[W
            int quizNum = quiz.QuizDatas.Count;
            
            WidPackage pack = new WidPackage();
            try
            {
                pack.Read(GetFileResource("Package1"));
            }
            catch (Exception)
            {
                MessageBox.Show("\[Xǂݍ߂܂B:" + "Package1");
                Application.Exit();
            }

            EventData eve = pack.Events[0];
            ContentData start = eve.Starts[0];
            start.Children.Clear();

            // ContentType.SubstituteStep
            ContentData cpRnd10_100 = new ContentData();
            cpRnd10_100.Kind = (byte)ContentType.SubstituteStep;
            cpRnd10_100.Version = VERSION;
            cpRnd10_100.Data = new BaseContent(ContentType.SubstituteStep);
            cpRnd10_100.Data.StringData = "??Random";
            cpRnd10_100.Data.StringData2 = "100̈";
            ContentData cpRnd10_10 = new ContentData();
            cpRnd10_10.Kind = (byte)ContentType.SubstituteStep;
            cpRnd10_10.Version = VERSION;
            cpRnd10_10.Data = new BaseContent(ContentType.SubstituteStep);
            cpRnd10_10.Data.StringData = "??Random";
            cpRnd10_10.Data.StringData2 = "10̈";
            ContentData cpRnd10_1 = new ContentData();
            cpRnd10_1.Kind = (byte)ContentType.SubstituteStep;
            cpRnd10_1.Version = VERSION;
            cpRnd10_1.Data = new BaseContent(ContentType.SubstituteStep);
            cpRnd10_1.Data.StringData = "??Random";
            cpRnd10_1.Data.StringData2 = "1̈";

            // Xebv@100̈
            ContentData x1 = new ContentData();
            x1.Kind = 0x11;
            x1.Version = VERSION;
            x1.Data = new BaseContent((ContentType)0x11);
            x1.Data.StringData = "100̈";

            for (int rnd_100 = 0; rnd_100 < 10; rnd_100++)
            {
                // Xebv@10̈
                ContentData y1 = new ContentData();
                y1.Kind = 0x11;
                y1.Choice = rnd_100.ToString();
                y1.Version = VERSION;
                y1.Data = new BaseContent(ContentType.BranchMultiStep);
                y1.Data.StringData = "10̈";

                for (int rnd_10 = 0; rnd_10 < 10; rnd_10++)
                {
                    // Xebv@1̈
                    ContentData z1 = new ContentData();
                    z1.Kind = 0x11;
                    z1.Choice = rnd_10.ToString();
                    z1.Version = VERSION;
                    z1.Data = new BaseContent(ContentType.BranchMultiStep);
                    z1.Data.StringData = "1̈";

                    ContentData cont;
                    ContentData oldCont;

                    for (int rnd_1 = 0; rnd_1 < 10; rnd_1++)
                    {
                        
                        int val = rnd_100 * 100 + rnd_10 * 10 + rnd_1;
                        if (quizNum * (MAX_QUIZ_NUM / quizNum) <= val)
                        {
                            // ]͂蒼
                            ContentData startLink = new ContentData();
                            startLink.Version = VERSION;
                            // X^[gN
                            startLink.Kind = 0x01;
                            startLink.Choice = rnd_1.ToString();
                            startLink.Data = new BaseContent(ContentType.LinkStart);
                            startLink.Data.StringData = start.Choice;

                            z1.Children.Add(startLink);
                        }
                        else
                        {
                            val = val % quizNum;
                            ContentData step = new ContentData();
                            step.Version = VERSION;
                            // XebvύX
                            step.Kind = 0x12;
                            step.Choice = rnd_1.ToString();
                            step.Data = new BaseContent(ContentType.SetStep);
                            step.Data.UInt = (uint)(val / 100);
                            step.Data.StringData = "100̈";
                            oldCont = step;

                            cont = new ContentData();
                            cont.Version = VERSION;
                            // XebvύX
                            cont.Kind = 0x12;
                            cont.Data = new BaseContent(ContentType.SetStep);
                            cont.Data.UInt = (uint)((val % 100) / 10);
                            cont.Data.StringData = "10̈";

                            oldCont.Children.Add(cont);
                            oldCont = cont;

                            cont = new ContentData();
                            cont.Version = VERSION;
                            // XebvύX
                            cont.Kind = 0x12;
                            cont.Data = new BaseContent(ContentType.SetStep);
                            cont.Data.UInt = (uint)(val % 10);
                            cont.Data.StringData = "1̈";

                            oldCont.Children.Add(cont);
                            oldCont = cont;

                            z1.Children.Add(step);
                        }
                    }
                    y1.Children.Add(z1);
                }
                x1.Children.Add(y1);
            }

            cpRnd10_1.Children.Add(x1);
            cpRnd10_10.Children.Add(cpRnd10_1);
            cpRnd10_100.Children.Add(cpRnd10_10);

            start.Children.Add(cpRnd10_100);

            pack.Write(scenaFolder + "\\Package1.wid");

        }

        private void MakeQuiz(string scenaFolder)
        {
            string format = "D3";
            // \pbP[W
            // `FbNpbP[W
            WidPackage pack = new WidPackage();
            try
            {
                pack.Read(GetFileResource("Package3"));
            }
            catch (Exception)
            {
                MessageBox.Show("\[Xǂݍ߂܂B:" + "Package3");
                Application.Exit();
            }

            WidPackage packCorr = new WidPackage();
            try
            {
                packCorr.Read(GetFileResource("Package4"));
            }
            catch (Exception)
            {
                MessageBox.Show("\[Xǂݍ߂܂B:" + "Package4");
                Application.Exit();
            }

            EventData eve = pack.Events[0];
            ContentData start = eve.Starts[0];
            start.Children.Clear();

            EventData eveCorr = packCorr.Events[0];
            ContentData startCorr = eveCorr.Starts[0];
            startCorr.Children.Clear();

            // Xebv@100̈
            ContentData x1 = new ContentData();
            x1.Kind = 0x11;
            x1.Version = VERSION;
            x1.Data = new BaseContent(ContentType.BranchMultiStep);
            x1.Data.StringData = "100̈";

            // Xebv@100̈
            ContentData x2 = new ContentData();
            x2.Kind = 0x11;
            x2.Version = VERSION;
            x2.Data = new BaseContent(ContentType.BranchMultiStep);
            x2.Data.StringData = "100̈";

            for (int rnd_100 = 0; rnd_100 < 10; rnd_100++)
            {

                // Xebv@10̈
                ContentData y1 = new ContentData();
                y1.Kind = 0x11;
                y1.Version = VERSION;
                y1.Choice = rnd_100.ToString();
                y1.Data = new BaseContent(ContentType.BranchMultiStep);
                y1.Data.StringData = "10̈";

                // Xebv@10̈
                ContentData y2 = new ContentData();
                y2.Kind = 0x11;
                y2.Version = VERSION;
                y2.Choice = rnd_100.ToString();
                y2.Data = new BaseContent(ContentType.BranchMultiStep);
                y2.Data.StringData = "10̈";

                for (int rnd_10 = 0; rnd_10 < 10; rnd_10++)
                {

                    // Xebv@1̈
                    ContentData z1 = new ContentData();
                    z1.Kind = 0x11;
                    z1.Version = VERSION;
                    z1.Choice = rnd_10.ToString();
                    z1.Data = new BaseContent(ContentType.BranchMultiStep);
                    z1.Data.StringData = "1̈";

                    // Xebv@1̈
                    ContentData z2 = new ContentData();
                    z2.Kind = 0x11;
                    z2.Version = VERSION;
                    z2.Choice = rnd_10.ToString();
                    z2.Data = new BaseContent(ContentType.BranchMultiStep);
                    z2.Data.StringData = "1̈";

                    for (int rnd_1 = 0; rnd_1 < 10; rnd_1++)
                    {
                        int val = rnd_100 * 100 + rnd_10 * 10 + rnd_1;

                        if(quiz.QuizDatas.Count > val){

                            GotakuQuestion question = quiz.QuizDatas[val];

                            // 胁bZ[WpwiύX
                            ContentData viewChange = new ContentData();
                            viewChange.Version = VERSION;
                            viewChange.Kind = VIEWCHANGE;
                            viewChange.Choice = rnd_1.ToString();
                            viewChange.Data = new ViewChangeContent();
                            ViewChangeContent dataPart = (ViewChangeContent)viewChange.Data;

                            CellData cell = new CellData();
                            cell.Version = VERSION;
                            cell.FileName = "Black.bmp";
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.MaskOn = true;
                            cell.FlagName = null;
                            cell.Visible = true;

                            dataPart.Cells.Add(cell);

                            string filename = @"jptx\sp" + val.ToString(format) + ".jptx";
                            cell = new CellData();
                            cell.Version = VERSION;
                            cell.FileName = filename;
                            cell.Left = 90;
                            cell.Top = 0;
                            cell.Width = 542;
                            cell.Height = 420;
                            cell.MaskOn = true;
                            cell.FlagName = null;
                            cell.Visible = true;

                            dataPart.Cells.Add(cell);

                            // JPTX쐬
                            List<int> indexes = GetRandomIndexes();
                            int correctNo = indexes.IndexOf(0);

                            StringBuilder quizDatas = new StringBuilder();

                            // ԍʂԍł͂ȂAW@{@WCfbNX
                            // ŕ\
                            //quizDatas.AppendLine(" " + (val + 1).ToString(format));
                            quizDatas.AppendLine(quiz.GetGenreName(val) + "  " +
                                (quiz.GetGenreIndex(val) + 1).ToString(format));

                            //quizDatas.AppendLine(question.Question);
                            string questionStr = question.Question;
                            int lineSize = 40;
                            for (int i = 0; i <= (questionStr.Length - 1) / lineSize; i++)
                            {
                                int length = Math.Min((i + 1) * lineSize, questionStr.Length) - i * lineSize;
                                quizDatas.AppendLine(questionStr.Substring(i * lineSize, length));
                            }

                            quizDatas.AppendLine();

                            for (int i = 0; i < 5; i++)
                            {
                                if (question.IsHorizontalChoice)
                                {
                                    quizDatas.Append((i + 1) + "." + question.Answer[indexes[i]] + " ");
                                }
                                else
                                {
                                    quizDatas.AppendLine((i + 1) + "." + question.Answer[indexes[i]]);
                                }
                            }

                            if (question.IsHorizontalChoice)
                                quizDatas.AppendLine();

                            MakeJptx(scenaFolder + "\\" + filename, 542, 420, 10, "FFFFFF", 
                                quizDatas.ToString());

                            z1.Children.Add(viewChange);


                            // Xebv@I
                            ContentData branch = new ContentData();
                            branch.Kind = 0x11;
                            branch.Version = VERSION;
                            branch.Choice = rnd_1.ToString();
                            branch.Data = new BaseContent(ContentType.BranchMultiStep);
                            branch.Data.StringData = "I";

                            // 
                            // wiύX
                            ContentData viewCorr = new ContentData();
                            viewCorr.Version = VERSION;
                            viewCorr.Kind = VIEWCHANGE;
                            viewCorr.Choice = correctNo.ToString();
                            viewCorr.Data = new ViewChangeContent();
                            ViewChangeContent dataCorrPart = (ViewChangeContent)viewCorr.Data;

                            // wiiՂ̍j
                            cell = new CellData();
                            cell.Version = VERSION;
                            cell.FileName = "Black.bmp";
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.MaskOn = true;
                            cell.FlagName = null;
                            cell.Visible = true;

                            dataCorrPart.Cells.Add(cell);

                            // 
                            filename = @"jptx\sp" + val.ToString(format) + ".jptx";
                            cell = new CellData();
                            cell.Version = VERSION;
                            cell.FileName = filename;
                            cell.Left = 90;
                            cell.Top = 0;
                            cell.Width = 542;
                            cell.Height = 420;
                            cell.MaskOn = true;
                            cell.FlagName = null;
                            cell.Visible = true;

                            dataCorrPart.Cells.Add(cell);

                            // I
                            filename = @"jptx\corr.jptx";
                            cell = new CellData();
                            cell.Version = VERSION;
                            cell.FileName = filename;
                            cell.Left = 10;
                            cell.Top = 72;
                            cell.Width = 80;
                            cell.Height = 30;
                            cell.MaskOn = true;
                            cell.FlagName = null;
                            cell.Visible = true;

                            dataCorrPart.Cells.Add(cell);

                            if (question.Expression != null)
                            {

                                // 
                                filename = @"jptx\sp" + val.ToString(format) + "_exp.jptx";
                                cell = new CellData();
                                cell.Version = VERSION;
                                cell.FileName = filename;
                                cell.Left = 90;
                                cell.Top = 200;
                                cell.Width = 542;
                                cell.Height = 420;
                                cell.MaskOn = true;
                                cell.FlagName = null;
                                cell.Visible = true;

                                dataCorrPart.Cells.Add(cell);

                                // jptx 
                                quizDatas = new StringBuilder();
                                quizDatas.Append(question.Expression);

                                MakeJptx(scenaFolder + "\\" + filename, 542, 420, 10, "FFFFFF",
                                    quizDatas.ToString());

                            }

                            // ʉ
                            ContentData wavCorr = new ContentData();
                            wavCorr.Kind = 0x09;
                            wavCorr.Version = VERSION;
                            wavCorr.Data = new BaseContent(ContentType.PlaySound);
                            wavCorr.Data.StringData = "VXeEVOi.wav";
                            viewCorr.Children.Add(wavCorr);

                            branch.Children.Add(viewCorr);

                            // 듚
                            // wiύX
                            ContentData viewMiss = new ContentData();
                            viewMiss.Version = VERSION;
                            viewMiss.Kind = VIEWCHANGE;
                            viewMiss.Choice = "Default";
                            viewMiss.Data = new ViewChangeContent();
                            ViewChangeContent dataMissPart = (ViewChangeContent)viewMiss.Data;

                            // wiiՂ̍j
                            cell = new CellData();
                            cell.Version = VERSION;
                            cell.FileName = "Black.bmp";
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.MaskOn = true;
                            cell.FlagName = null;
                            cell.Visible = true;

                            dataMissPart.Cells.Add(cell);

                            // 
                            filename = @"jptx\sp" + val.ToString(format) + ".jptx";
                            cell = new CellData();
                            cell.Version = VERSION;
                            cell.FileName = filename;
                            cell.Left = 90;
                            cell.Top = 0;
                            cell.Width = 542;
                            cell.Height = 420;
                            cell.MaskOn = true;
                            cell.FlagName = null;
                            cell.Visible = true;

                            dataMissPart.Cells.Add(cell);

                            // 듚
                            filename = @"jptx\miss" + (correctNo + 1) + ".jptx";
                            cell = new CellData();
                            cell.Version = VERSION;
                            cell.FileName = filename;
                            cell.Left = 10;
                            cell.Top = 72;
                            cell.Width = 80;
                            cell.Height = 30;
                            cell.MaskOn = true;
                            cell.FlagName = null;
                            cell.Visible = true;

                            dataMissPart.Cells.Add(cell);

                            if (question.Expression != null)
                            {
                                // 
                                filename = @"jptx\sp" + val.ToString(format) + "_exp.jptx";
                                cell = new CellData();
                                cell.Version = VERSION;
                                cell.FileName = filename;
                                cell.Left = 90;
                                cell.Top = 200;
                                cell.Width = 542;
                                cell.Height = 420;
                                cell.MaskOn = true;
                                cell.FlagName = null;
                                cell.Visible = true;

                                dataMissPart.Cells.Add(cell);
                            }

                            //듚 ʉ
                            ContentData wavMiss = new ContentData();
                            wavMiss.Kind = 0x09;
                            wavMiss.Version = VERSION;
                            wavMiss.Data = new BaseContent(ContentType.PlaySound);
                            wavMiss.Data.StringData = "VXeEG[.wav";
                            viewMiss.Children.Add(wavMiss);

                            branch.Children.Add(viewMiss);

                            z2.Children.Add(branch);
                        }
                    }

                    y1.Children.Add(z1);
                    y2.Children.Add(z2);
                }

                x1.Children.Add(y1);
                x2.Children.Add(y2);

            }

            start.Children.Add(x1);
            startCorr.Children.Add(x2);

            pack.Write(scenaFolder + "\\Package3.wid");
            packCorr.Write(scenaFolder + "\\Package4.wid");

        }

        private List<int> GetRandomIndexes()
        {
            int seed = Environment.TickCount;
            Random rnd = new Random(seed++);
            List<int> ret = new List<int>();
            for (int i = 0; i < 5; i++)
                ret.Add(i);

            for (int count = 0; count < 1000; count++)
            {
                if (count % 100 == 0) rnd = new Random(seed++);

                int prev = rnd.Next(5);
                int next = rnd.Next(5);
                int temp = ret[prev];
                ret[prev] = ret[next];
                ret[next] = temp;
            }
            return ret;
        }

        private void MakeArea(string scenaName, string scenaFolder)
        {
            string scenaJptxFolder = scenaFolder + "\\jptx";
            MakeJptx(scenaJptxFolder + "\\title.jptx", 632, 420, 36, "111111", scenaName);
            
            WidArea area = new WidArea();
            try
            {
                area.Read(GetFileResource("Area1"));
            }
            catch (Exception)
            {
                MessageBox.Show("\[Xǂݍ߂܂B:" + "Area1");
                Application.Exit();
            }

            area.Write(scenaFolder + "\\Area1.wid");

            WidArea area2 = new WidArea();
            try
            {
                area2.Read(GetFileResource("Area2"));
            }
            catch (Exception)
            {
                MessageBox.Show("\[Xǂݍ߂܂B:" + "Area2");
                Application.Exit();
            }
            area2.Write(scenaFolder + "\\Area2.wid");
        }

        private void MakeWsm(string scenaName, string scenaFolder)
        {
            SummaryData summary = new SummaryData();

            try
            {
                summary.Read(GetFileResource("Summary"));
            }
            catch (Exception)
            {
                MessageBox.Show("\[Xǂݍ߂܂B:" + "Summary");
                Application.Exit();
            }

            summary.ScenarioName = scenaName;
            summary.Author = " ";

            summary.Write(scenaFolder + "\\Summary.wsm");

        }

        private void MakeJptx(string filePath, int  width, int height, int fontPixel,string fontColor, string data)
        {
            StringBuilder contents = new StringBuilder();

            try
            {

                /************ jptx L *************************
                [jptx:init]
                backwidth=632
                backheight=420
                lineheight=120
                backcolor=$000000
                autoline=1
                antialias=1
                fonttransparent=0
                fontpixels=36
                fontface=lr 
                fontcolor=$10EFEF
                [jptx:begin]
                Ts`jt
                [jptx:end]
                ********************************************/

                contents.AppendLine("[jptx:init]");
                contents.AppendLine("backwidth=" + width.ToString());
                contents.AppendLine("backheight=" + height.ToString());

                // s(%)
                contents.AppendLine("lineheight=120");

                // }XN̂ŏdȂȂFȂ牽ł
                contents.AppendLine("backcolor=$000000");

                // s
                contents.AppendLine("autoline=1");
                // tHgGCAX
                contents.AppendLine("antialias=1");
                // F𓧖F
                contents.AppendLine("fonttransparent=0");

                // tHgTCY
                contents.AppendLine("fontpixels=" + fontPixel.ToString());

                // tHg
                contents.AppendLine("fontface=lr ");

                // tHg̕WF
                contents.AppendLine("fontcolor=$" + fontColor);

                contents.AppendLine("[jptx:begin]");

                contents.AppendLine(data);

                contents.AppendLine("[jptx:end]");

                File.WriteAllText(filePath, contents.ToString(), Encoding.GetEncoding(932));
            }
            catch (Exception)
            {
                MessageBox.Show("jptxt@C̏oɎsB");
                Application.Exit();
            }

        }

        /// <summary>
        /// hbOG^[Cxg
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Compiler_DragEnter(object sender, DragEventArgs e)
        {

            //hbOĂf[^ DataFormats.FileDrop ׁA
            //ł΃hbvʂAllɂ
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effect = DragDropEffects.All;
        }

        /// <summary>
        /// hbOhbvCxg
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Compiler_DragDrop(object sender, DragEventArgs e)
        {
            //hbvꂽf[^string^ׂ
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                // Ώ
                string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
                if (files.Length == 1 && File.Exists(files[0]))
                {
                    this.filename = files[0];
                    OpenProcess();
                }
            }
        }

    }

}